@@ -1,5 +1,5 @@
 package ETL::Yertl::Command::yq;
-our $VERSION = '0.037';
+our $VERSION = '0.038';
 # ABSTRACT: Filter and construct documents using a mini-language
 
 use ETL::Yertl;
@@ -7,6 +7,9 @@ use ETL::Yertl::Util qw( load_module );
 use boolean qw( :all );
 use Module::Runtime qw( use_module );
 our $VERBOSE = $ENV{YERTL_VERBOSE} // 0;
+use IO::Async::Loop;
+use ETL::Yertl::Format;
+use ETL::Yertl::FormatStream;
 
 sub is_empty {
     return !$_[0] || ref $_[0] eq 'empty';
@@ -24,30 +27,44 @@ sub main {
 
     die "Must give a filter\n" unless $filter;
 
+    my $loop = IO::Async::Loop->new;
+
     push @files, "-" unless @files;
     for my $file ( @files ) {
+
         # We're doing a similar behavior to <>, but manually for easier testing.
-        my $fh;
+        my $scope = {};
+        my %args = (
+            on_doc => sub {
+                my ( $self, $doc, $eof ) = @_;
+                #; say STDERR "Got doc: " . $doc;
+                return unless $doc; # XXX: This shouldn't be happening, but it is
+                my @output = $class->filter( $filter, $doc, $scope );
+                $class->write( \@output, \%opt );
+            },
+            on_read_eof => sub {
+                # Finish the scope, cleaning up any collections
+                $class->write( [ $class->finish( $scope ) ], \%opt );
+                $loop->stop;
+            },
+        );
+        my $in;
         if ( $file eq '-' ) {
-            # Use the existing STDIN so tests can fake it
-            $fh = \*STDIN;
+            $in = ETL::Yertl::FormatStream->new_for_stdin( %args );
         }
         else {
-            unless ( open $fh, '<', $file ) {
+            open my $fh, '<', $file or do {
                 warn "Could not open file '$file' for reading: $!\n";
                 next;
-            }
+            };
+            $in = ETL::Yertl::FormatStream->new(
+                read_handle => $fh,
+                %args,
+            );
         }
+        $loop->add( $in );
+        $loop->run;
 
-        my $in_fmt = load_module( format => 'default' )->new( input => $fh );
-        my $scope = {};
-        for my $doc ( $in_fmt->read ) {
-            my @output = $class->filter( $filter, $doc, $scope );
-            $class->write( \@output, \%opt );
-        }
-
-        # Finish the scope, cleaning up any collections
-        $class->write( [ $class->finish( $scope ) ], \%opt );
     }
 }
 
@@ -59,17 +76,17 @@ sub write {
         return;
     }
 
-    my $out_fmt = load_module( format => 'default' )->new;
+    my $format = ETL::Yertl::Format->get_default;
     for my $doc ( @$docs ) {
         next if is_empty( $doc );
         if ( isTrue( $doc ) ) {
-            print $out_fmt->write( "true" );
+            print $format->format( "true" );
         }
         elsif ( isFalse( $doc ) ) {
-            print $out_fmt->write( "false" );
+            print $format->format( "false" );
         }
         else {
-            print $out_fmt->write( $doc );
+            print $format->format( $doc );
         }
     }
 }
@@ -105,7 +122,7 @@ ETL::Yertl::Command::yq - Filter and construct documents using a mini-language
 
 =head1 VERSION
 
-version 0.037
+version 0.038
 
 =head1 SYNOPSIS
 
